#include "headers.h"
#include "cat.h"

Cat::Cat()
{
	InitializeCriticalSection(&cs);
	Reset();
}

Cat::~Cat()
{
	DeleteCriticalSection(&cs);
}

void Cat::Reset()
{
	x		= 0;
	y		= 0;
	valid	= false;
	buttons	= 0;
}

void Cat::Open(int _sensitivity, LOGCONTEXT _lc)
{
	Reset();
	lc = _lc;
	sensitivity = _sensitivity;
}

uint Cat::InBx(uint port)
{
	uint ret = 0xff;

	switch(port)
	{
	case 0xb0:
		ret = valid && sv ? Limit((x - sx) * sensitivity / 100, 127, -128) : 0;
		break;
	case 0xb1:
		ret = valid && sv ? Limit((sy - y) * sensitivity / 100, 127, -128) : 0;
		break;
	case 0xb2:
		EnterCriticalSection(&cs);
		sx = x;
		sy = y;
		sv = valid;
		LeaveCriticalSection(&cs);
		break;
	case 0xb3:
		ret = valid ? ((buttons & 1) << 4) | ((buttons & 2) << 2) | (buttons & 4) : 0;
		break;
	}
	return ret;
}

void Cat::SetValue(PACKET& packet, bool _valid)
{
	EnterCriticalSection(&cs);

	valid	= _valid;
	x		= packet.pkX;
	y		= packet.pkY;
	buttons	= packet.pkButtons;

	LeaveCriticalSection(&cs);
}
