#include "headers.h"
#include "config.h"
#include "resource.h"

// ---------------------------------------------------------------------------

ConfigTablet::ConfigTablet()
{
	gate.SetDestination(PageGate, this);
}

bool ConfigTablet::Init(HINSTANCE _hinst, Tablet* _tablet)
{
	hinst = _hinst;
	tablet = _tablet;

	char	szBuf[_MAX_PATH];

	GetModuleFileName(0, szBuf, _MAX_PATH);
	LPTSTR filepart;
	GetFullPathName(szBuf, _MAX_PATH, szIniFile, &filepart);
	*filepart = '\0';
	lstrcpy(filepart, "M88.INI");

	config.mode       = GetPrivateProfileInt("PC8875", "MODE",       MODE_DEFAULT, szIniFile);
	config.usekey1    = GetPrivateProfileInt("PC8875", "USEKEY1",    USEKEY1_DEFAULT, szIniFile);
	config.keyport1   = GetPrivateProfileInt("PC8875", "KEYPORT1",   KEYPORT1_DEFAULT, szIniFile);
	config.keybit1    = GetPrivateProfileInt("PC8875", "KEYBIT1",    KEYBIT1_DEFAULT, szIniFile);
	config.usekey2    = GetPrivateProfileInt("PC8875", "USEKEY2",    USEKEY2_DEFAULT, szIniFile);
	config.keyport2   = GetPrivateProfileInt("PC8875", "KEYPORT2",   KEYPORT2_DEFAULT, szIniFile);
	config.keybit2    = GetPrivateProfileInt("PC8875", "KEYBIT2",    KEYBIT2_DEFAULT, szIniFile);
	config.mouse_sens = GetPrivateProfileInt("PC8875", "MOUSE_SENS", MOUSE_SENS_DEFAULT, szIniFile);
	config.mode       = Limit(config.mode,       8,  0);
	config.usekey1    = Limit(config.usekey1,    1,  0);
	config.keyport1   = Limit(config.keyport1,   14, 0);
	config.keybit1    = Limit(config.keybit1,    7,  0);
	config.usekey2    = Limit(config.usekey2,    1,  0);
	config.keyport2   = Limit(config.keyport2,   14, 0);
	config.keybit2    = Limit(config.keybit2,    7,  0);
	config.mouse_sens = Limit(config.mouse_sens, MOUSE_SENS_MAX,  MOUSE_SENS_MIN);

	return true;
}

bool IFCALL ConfigTablet::Setup(IConfigPropBase* _base, PROPSHEETPAGE* psp)
{
	base = _base;
	
	memset(psp, 0, sizeof(PROPSHEETPAGE));
	psp->dwSize = sizeof(PROPSHEETPAGE);
	psp->dwFlags = 0;
	psp->hInstance = hinst;
	psp->pszTemplate = MAKEINTRESOURCE(IDD_CONFIG);
	psp->pszIcon = 0;
	psp->pfnDlgProc = (DLGPROC) (void*) gate;
	psp->lParam = 0;
	return true;
}

BOOL ConfigTablet::PageProc(HWND hdlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		InitDialog(hdlg);
		return TRUE;
	case WM_NOTIFY:
		switch (((NMHDR*) lp)->code)
		{
		case PSN_SETACTIVE:
			Update(hdlg);
			base->PageSelected(this);
			break;
		case PSN_APPLY:
			Apply(hdlg);
			base->Apply();
			return PSNRET_NOERROR;
		}
		return TRUE;
	case WM_COMMAND:
		return Command(hdlg, HWND(lp), HIWORD(wp), LOWORD(wp));
	case WM_HSCROLL:
		UpdateSlider(hdlg);
		base->PageChanged(hdlg);
		return TRUE;
	}
	return FALSE;
}

BOOL CALLBACK ConfigTablet::PageGate
(ConfigTablet* config, HWND hwnd, UINT m, WPARAM w, LPARAM l)
{
	return config->PageProc(hwnd, m, w, l);
}

void ConfigTablet::InitDialog(HWND hdlg)
{
	config_disp = config;

	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"Disable");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"NEC PC-8875 (Ascii binary)");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"NEC PC-8875 (BCD character)");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"Logitec MYPAD-A3 K-510");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"WACOM CREATOR WT-3000");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"EPSON MT-20 (TABLET)");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"EPSON MT-20 (MOUSE)");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"HAL Laboratory CAT-8800");
	SendDlgItemMessage(hdlg, IDC_MODE, CB_ADDSTRING, 0, (LPARAM)"BUS MOUSE");
	SendDlgItemMessage(hdlg, IDC_MOUSE_SLIDER, TBM_SETRANGE, TRUE, MAKELONG(MOUSE_SENS_MIN, MOUSE_SENS_MAX));
	SendDlgItemMessage(hdlg, IDC_MOUSE_SLIDER, TBM_SETPOS, FALSE, config_disp.mouse_sens);
}

void ConfigTablet::Update(HWND hdlg)
{
	SendDlgItemMessage(hdlg, IDC_MODE, CB_SETCURSEL, config_disp.mode, 0);
	SetDlgItemText(hdlg, IDC_STATUS, tablet->GetMessage());
	EnableSlider(hdlg);
	UpdateSlider(hdlg);
}

void ConfigTablet::UpdateSlider(HWND hdlg)
{
	char buf[16];
	config_disp.mouse_sens = SendDlgItemMessage(hdlg, IDC_MOUSE_SLIDER, TBM_GETPOS, 0, 0);
	sprintf(buf, "%d%%", config_disp.mouse_sens);
	SetDlgItemText(hdlg, IDC_MOUSE, buf);
}

void ConfigTablet::EnableSlider(HWND hdlg)
{
	SendDlgItemMessage(hdlg, IDC_MOUSE_SLIDER, TBM_SETRANGE, TRUE, MAKELONG(MOUSE_SENS_MIN, MOUSE_SENS_MAX));
	BOOL use_mouse = (config_disp.mode == Tablet::TabletMode::mouse || 
					  config_disp.mode == Tablet::TabletMode::hal ||
					  config_disp.mode == Tablet::TabletMode::busmouse) ? TRUE : FALSE;
	EnableWindow(GetDlgItem(hdlg, IDC_MOUSE_SLIDER), use_mouse);
	EnableWindow(GetDlgItem(hdlg, IDC_MOUSE), use_mouse);
}

void ConfigTablet::Apply(HWND hdlg)
{
	config = config_disp;

	WritePrivateProfileInt("MODE",       config.mode);
	WritePrivateProfileInt("USEKEY1",    config.usekey1);
	WritePrivateProfileInt("KEYPORT1",   config.keyport1);
	WritePrivateProfileInt("KEYBIT1",    config.keybit1);
	WritePrivateProfileInt("USEKEY2",    config.usekey2);
	WritePrivateProfileInt("KEYPORT2",   config.keyport2);
	WritePrivateProfileInt("KEYBIT2",    config.keybit2);
	WritePrivateProfileInt("MOUSE_SENS", config.mouse_sens);
	
	if (tablet) 
		tablet->Open(&config);
	SetDlgItemText(hdlg, IDC_STATUS, tablet->GetMessage());
}

void ConfigTablet::WritePrivateProfileInt(char* keyname, int keyvalue)
{
	char	buf[16];
	sprintf(buf, "%d", keyvalue);
	WritePrivateProfileString("PC8875", keyname, buf, szIniFile);
}


BOOL ConfigTablet::Command(HWND hdlg, HWND hwctl, UINT nc, UINT id)
{
	switch (id)
	{
	case IDC_MODE:
		if (nc == CBN_SELCHANGE)
		{
			int baud = SendDlgItemMessage(hdlg, IDC_MODE, CB_GETCURSEL, 0, 0);
			if (baud != config_disp.mode)
				base->PageChanged(hdlg);
			config_disp.mode = baud;
			EnableSlider(hdlg);
			return true;
		}
		break;
	}
	return FALSE;
}
