#ifndef INCL_PC8875_H
#define INCL_PC8875_H

#include "device.h"
#include "misc.h"
#include "if/ifcommon.h"

// ---------------------------------------------------------------------------

class Pc8875
{
public:
	Pc8875();
	~Pc8875();

	enum Format { ascii_binary, bcd };
	
	bool Init();
	void Release();

	void Reset();
	uint Out20(uint, byte* buf);
	uint SetValue(byte* buf, PACKET& packet, bool valid, DWORD tick);

	bool Open(Format format, LOGCONTEXT lc);
	void Close();
	void SetStatusMessage(char* message);

private:
	enum TransferMode { single=0, multiple };
	enum ReadMode { point=0, switch_stream, presence_stream, increment };
	enum ReadStatus { stop, collect, wait, swoff, oarea };

	uint SetAsciiBinaryCommand(uint data, byte* buf);
	uint SetBcdCommand(uint data, byte* buf);
	void SetBcdCommandSub(ReadMode rm, uint data, uint top, uint& len, byte* buf);
	void ChangeTransferMode(TransferMode tm);
	uint ChangeReadMode(ReadMode rm, byte* buf);
	void SetData(byte* data, uint count);
	uint SetValueString(byte* buf, int x, int y, bool pen, bool valid);
	bool SetPointValue(int x, int y, bool pen, bool valid);
	bool SetSwitchStreamValue(int x, int y, bool pen, bool valid, DWORD tick);
	bool SetPresenceStreamValue(int x, int y, bool pen, bool valid, DWORD tick);

	LOGCONTEXT	lc;
	Format		format;
	TransferMode transfermode;
	ReadMode	readmode;
	int			transferrate;
	int			command;
	int			valuemode;
	int			last_x;
	int			last_y;
	bool		last_pen;
	bool		last_valid;
	int			send_x;
	int			send_y;
	bool		send_pen;
	bool		invalid_oarea;
	bool		point_pen;
	DWORD		last_tick;
	int			readstatus;
};

#endif // INCL_PC8875_H
